@@ -85,6 +85,8 @@ if ($sim_mode) {
   print "Simulation mode. Just printing what would be done.\n";
 }
 
+local $SIG{CHLD} = 'IGNORE'; # Auto reap children
+
 #############################################################################
 ### configuration ###########################################################
 # Include the library for the config file parser
@@ -105,6 +107,8 @@ if ( $cfg->{clean_cache} ne 1 ) {
     exit 0;
 }
 
+check_install(); # Before we give up rights
+
 # change uid and gid if root and another user/group configured
 if (($cfg->{user} && $cfg->{user} !~ 'root' && !$> )
     || ($cfg->{group} && $cfg->{group} !~ 'root' && !$) =~ /^0/)){
@@ -137,8 +141,16 @@ sub printmsg {
 sub open_lock {
     my ($file) = @_;
 
+    my $retried;
+  TRY:
     #  Lock header LOCK_EX first to block apt-cacher from working on it
     open(my $hfh, '+<', "../headers/$file") || do {
+	if ($!{ENOENT} && !$retried && !$offline) {
+	    printmsg ("../headers/$file missing. Attempting download\n");
+	    get($file);
+	    $retried = 1;
+	    goto TRY;
+	}
 	warn ("Error: cannot open ../headers/$file for locking: $!\n");
 	return;
     };
@@ -714,7 +726,12 @@ if ($cfg->{checksum}) {
 { # Scoping block
     # Build a source package version reverse hash for changelog validation from the .dsc files
     printmsg "Building source package file/version table\n";
-    my %svrhash = map {m#([-+.a-z0-9]+_(?:\d:)?[-+.~a-zA-Z0-9]+)\.dsc# && $1 => 1} keys %valid;
+
+    open(my $tmpfile, "+>", undef) or die $!;
+    tie my %svrhash, 'GDBM_File', fd_path($tmpfile), &GDBM_NEWDB|&GDBM_FAST, oct(600) # Does a separate open
+      or die "GDBM_File tie failed: $!";
+    close($tmpfile); # So we can close this
+    %svrhash = map {m#([-+.a-z0-9]+_(?:\d:)?[-+.~a-zA-Z0-9]+)\.dsc# && $1 => 1} keys %valid;
 
     foreach (glob('*{,/*}')) {
 	next if -d; # Skip directories
