@@ -52,7 +52,6 @@ sub read_config {
 		  generate_reports => 1,
 		  group => eval {my $g = $); $g =~ s/\s.*$//; $g},
 		  http_proxy => '',
-		  http_proxy_auth => '',
 		  libcurl_socket => '/var/run/apt-cacher/libcurl.socket',
 		  limit => 0,
 		  limit_global => 0,
@@ -66,6 +65,7 @@ sub read_config {
 						      amd64
 						      alpha
 						      arm
+						      arm64
 						      armel
 						      armhf
 						      hppa
@@ -81,10 +81,14 @@ sub read_config {
 						      netbsd-alpha
 						      netbsd-i386
 						      powerpc
+						      powerpcspe
+						      ppc64
 						      s390
 						      s390x
 						      sh4
 						      sparc
+						      sparc64
+						      x32
 						 )),
 		  ubuntu_release_names => join(', ', qw(
 							   dapper
@@ -105,9 +109,9 @@ sub read_config {
 							   saucy
 							   trusty
 							   utopic
+							   vivid
+							   wily
 						      )),
-		  use_proxy => 0,
-		  use_proxy_auth => 0,
 		  user => $>,
 
 		  # Private
@@ -313,12 +317,46 @@ sub private_config {
     }
 
     # Proxy support
-    foreach ('proxy', 'proxy_auth') {
-        if ($cfg->{"use_$_"} && !$cfg->{"http_$_"}) {
-	    warn "use_$_ specified without http_$_ being set. Disabling.";
-	    $cfg->{"use_$_"}=0;
+    for ($cfg->{http_proxy}) {
+	last unless length;
+
+	# Legacy use_proxy warning
+	if (exists $cfg->{use_proxy} && !$cfg->{use_proxy}) {
+	    warn 'Legacy use_proxy=0 is deprecated and is slated for removal. Unset http_proxy if upstream proxy not required';
+	    last; # But still honour it
 	}
+
+	$cfg->{_proxy} = URI->new((m!^[^:/?#]+://! ? '' : 'http://') . $_);
+	if ($cfg->{_proxy}) {
+	    # Clear any path
+	    $cfg->{_proxy}->path(undef);
+
+	    # Legacy proxy authorisation
+	    if ($cfg->{http_proxy_auth}) {
+
+		# Legacy use_proxy_auth warning
+		if (exists $cfg->{use_proxy_auth} && !$cfg->{use_proxy_auth}) {
+		    warn 'Legacy use_proxy_auth=0 is deprecated and is slated for removal. Unset http_proxy_auth if upstream proxy authentication not required';
+		}
+		else {
+		    if ($cfg->{_proxy}->userinfo) {
+			warn 'Upstream proxy authorisation already set, ignoring deprecated http_proxy_auth';
+		    }
+		    else {
+			$cfg->{_proxy}->userinfo($cfg->{http_proxy_auth});
+		    }
+		}
+	    }
+	}
+	else {
+	    info_message("Warning: failed to parse http_proxy $cfg->{http_proxy} as URI. Ignoring.");
+	}
+        # Hide auth details so they don't appear in /config
+	m%[^/?#]+(?=@)% && substr ($_, $-[0], $+[0] - $-[0]) =~ tr/:/*/c;
     }
+    # Hide legacy auth details so they don't appear in /config
+    $cfg->{http_proxy_auth} =~ tr/:/*/c if exists $cfg->{http_proxy_auth};
+
 
     # Rate limit and disk_usage_limit support
     foreach (qw(limit disk_usage_limit)) {
